<?php

declare(strict_types=1);

namespace Doctrine\Tests\Models\AbstractFetchEager;

use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity()
 */
class MobileRemoteControl extends AbstractRemoteControl
{
}
